
import argparse
from langchain_community.vectorstores import FAISS
from langchain_community.embeddings import OllamaEmbeddings
from langchain.chains import RetrievalQA
from langchain_community.llms import Ollama

class Query:
    def __init__(self, store_path="vector_store"):
        self.store_path = store_path
        self.embeddings = OllamaEmbeddings(model="llama3.1:8b")
        self.vector_store = self._load_store()
        if self.vector_store is None:
            raise FileNotFoundError(f"Vector store not found at {self.store_path}")
        self.llm = Ollama(model="llama3.1:8b")
        self.qa_chain = self._setup_qa_chain()

    def _load_store(self):
        try:
            return FAISS.load_local(self.store_path, self.embeddings, allow_dangerous_deserialization=True)
        except Exception as e:
            print(f"Error loading vector store: {e}")
            return None

    def _setup_qa_chain(self):
        retriever = self.vector_store.as_retriever()
        return RetrievalQA.from_chain_type(
            llm=self.llm,
            chain_type="stuff",
            retriever=retriever
        )

    def ask(self, question):
        return self.qa_chain.run(question)

def main():
    parser = argparse.ArgumentParser(description="Query the RAG model.")
    parser.add_argument("question", help="The question to ask.")
    parser.add_argument("--store", default="vector_store", help="Path to the vector store directory.")
    args = parser.parse_args()

    try:
        query_engine = Query(store_path=args.store)
        answer = query_engine.ask(args.question)
        print("Answer:", answer)
    except FileNotFoundError as e:
        print(e)

if __name__ == "__main__":
    main()
