
import os
import shutil
import pytest
from src.vector_store import VectorStore
from src.query import Query

@pytest.fixture(scope="module")
def test_data():
    if not os.path.exists("test_docs"):
        os.makedirs("test_docs")
    with open("test_docs/doc1.txt", "w") as f:
        f.write("This is a test document about apples.")
    with open("test_docs/doc2.txt", "w") as f:
        f.write("This is a test document about oranges.")
    yield
    shutil.rmtree("test_docs")
    if os.path.exists("test_store"):
        shutil.rmtree("test_store")

def test_vector_store(test_data):
    store = VectorStore(store_path="test_store")
    store.add_file("test_docs/doc1.txt")
    store.add_file("test_docs/doc2.txt")
    
    # Test that files are added
    assert os.path.exists("test_store")
    
    # Test that adding the same file again is skipped
    store.add_file("test_docs/doc1.txt") # This should be skipped

    # Verify content by querying
    query_engine = Query(store_path="test_store")
    answer = query_engine.ask("What are apples?")
    assert "apples" in answer.lower()

def test_query(test_data):
    # The vector store is already created by the previous test
    query_engine = Query(store_path="test_store")
    answer = query_engine.ask("What are oranges?")
    assert "oranges" in answer.lower()

def test_deduplication(test_data):
    store = VectorStore(store_path="test_store_dedup")
    
    # Add a file
    store.add_file("test_docs/doc1.txt")
    file_hash = store._get_file_hash("test_docs/doc1.txt")
    assert store.is_file_in_store(file_hash)

    # Try to add it again
    store.add_file("test_docs/doc1.txt")
    
    # We can't easily check the number of documents in FAISS directly without more complex logic,
    # but we can rely on the is_file_in_store check and the print statement for now.
    # A more robust test would involve mocking the vector store's add_documents method.
    
    shutil.rmtree("test_store_dedup")

if __name__ == "__main__":
    pytest.main()
