
import os
import argparse
from langchain_community.vectorstores import FAISS
from langchain_community.embeddings import OllamaEmbeddings
from langchain.text_splitter import RecursiveCharacterTextSplitter
from langchain_community.document_loaders import TextLoader
import hashlib

class VectorStore:
    def __init__(self, store_path="vector_store"):
        self.store_path = store_path
        self.embeddings = OllamaEmbeddings(model="llama3.1:8b")
        self.text_splitter = RecursiveCharacterTextSplitter(chunk_size=1000, chunk_overlap=200)
        self.vector_store = self._load_store()

    def _load_store(self):
        if os.path.exists(self.store_path):
            return FAISS.load_local(self.store_path, self.embeddings, allow_dangerous_deserialization=True)
        return None

    def _save_store(self):
        if self.vector_store:
            self.vector_store.save_local(self.store_path)

    def _get_file_hash(self, file_path):
        hasher = hashlib.md5()
        with open(file_path, 'rb') as f:
            buf = f.read()
            hasher.update(buf)
        return hasher.hexdigest()

    def add_file(self, file_path, parent_id=""):
        file_hash = self._get_file_hash(file_path)
        if self.is_file_in_store(file_hash):
            print(f"File {file_path} is already in the vector store. Skipping.")
            return

        loader = TextLoader(file_path)
        documents = loader.load()
        chunks = self.text_splitter.split_documents(documents)

        # Add file hash to metadata
        for chunk in chunks:
            chunk.metadata['file_hash'] = file_hash

        if self.vector_store is None:
            self.vector_store = FAISS.from_documents(chunks, self.embeddings)
        else:
            self.vector_store.add_documents(chunks)

        self._save_store()
        print(f"Added {file_path} to the vector store.")

    def is_file_in_store(self, file_hash):
        if self.vector_store is None:
            return False

        # This is a workaround to check for existence. A more robust solution
        # would be to store and query metadata directly if the vector store supports it.
        # For FAISS, we can iterate through the docstore.
        if hasattr(self.vector_store, 'docstore'):
            for doc_id in self.vector_store.docstore._dict:
                doc = self.vector_store.docstore._dict[doc_id]
                if doc.metadata.get('file_hash') == file_hash:
                    return True
        return False


def main():
    parser = argparse.ArgumentParser(description="Add text files to the vector store.")
    parser.add_argument("files", nargs="+", help="List of text files to add.")
    parser.add_argument("--store", default="vector_store", help="Path to the vector store directory.")
    args = parser.parse_args()

    vector_store = VectorStore(store_path=args.store)
    for file_path in args.files:
        if os.path.exists(file_path):
            vector_store.add_file(file_path)
        else:
            print(f"File not found: {file_path}")

if __name__ == "__main__":
    main()
