from pathlib import Path
import logging
from typing import Optional, List
from dataclasses import dataclass
import tempfile
import os
import ocrmypdf
from markitdown import MarkItDown, FileConversionException, UnsupportedFormatException

from gtagger.convert.converter import Convert, ConversionError,TikaConverter
from gtagger.config import get_config
from gtagger.file_explorer import FileExplorer
from gtagger.convert.conversion_result import ConversionResult
from gtagger.convert.handler import PostConversionHandler

log = logging.getLogger(__name__)
config = get_config()



def save_converted_file(file: Path, result: ConversionResult) -> None:
    target_file = file.with_suffix(".md")

    # TODO ATTENZIONE: qui si fa la black list per l'ia (es Rassegna Stampa")

    with open(target_file, "w") as f:
        f.write(result.content)
        log.info(f"{target_file} salvato con successo (metodo: {result.method})")


def do_markitdown(filepath: str):
    md = MarkItDown()
    log.info(f"Conversione del file {filepath}")
    try:
        result = md.convert(filepath)
    except FileConversionException as e:
        log.error(f"La conversione del file {filepath} ha prodotto l'errore: {e}.")
        return None
    except UnsupportedFormatException as e:
        log.error(f"Formato del file {filepath} non supportato.")
        return None
    except:
        log.error(
            f"La conversione del file {filepath} ha prodotto un errore non gestito"
        )
        return None
    return result

def convert_scanned_pdf(file: Path) -> ConversionResult:
    """
    Esegue OCR su PDF scansionato e restituisce un ConversionResult.
    
    """
    tmp_path = None
    try:
        log.info(f"OCR del file {file}")
        with tempfile.NamedTemporaryFile(mode="w+", delete=False) as tmp_file:
            tmp_path = tmp_file.name
        ocrmypdf.ocr(
            input_file=file,
            output_file=tmp_path,
            language="ita",
            force_ocr=True,
            max_image_mpixels=600,
        )
        
        log.debug(f"Creato file temporaneo OCR {tmp_path}")
        result = do_markitdown(Path(tmp_path))
        if result.text_content:
            return ConversionResult(success=True, content=result.text_content, method="ocr")
        return ConversionResult(success=False, error="OCR riuscito, ma estrazione vuota")
    
    except ocrmypdf.exceptions.TaggedPDFError as e:
        msg = f"File {file} marcato come tagged"
        log.error(msg)
        # Questa eccezione è sollevata se il PDF è marcato come TAGGED
        # --force-ocr per forzare
        return ConversionResult(success=False, error=msg)
    # Ho trovato un pdf che risulta di tipo "AppleDouble encoded Macintosh file"
    except ocrmypdf.exceptions.UnsupportedImageFormatError as e:
        # Non faccio il raise, altrimenti si bloccherà sempre (non credo si possa eliminare il documento)
        msg = f"Formato non supportato per {file}"
        log.error(msg)
        return ConversionResult(success=False, error=msg)
    except Exception as e:
        msg = f"Errore generico durante OCR di {file}: {e}"
        log.exception(msg)
        return ConversionResult(success=False, error=msg)
    finally:
        if tmp_path and os.path.exists(tmp_path):
            os.remove(tmp_path)
            log.debug(f"File temporaneo OCR eliminato: {tmp_path}")


def convert_file(file: Path, converters: List[Convert]) -> ConversionResult:
    for converter in converters:
        try:
            content = converter.extract_text(file)
            if content:
                return ConversionResult(True, content = content, method=converter.name)
        except ConversionError as e:
            log.warning(f"Conversione con {converter.name} su {file} fallita: {e}")
            continue
            
    return ConversionResult(False, error="tutti i convertitori hanno fallito")
        


# Qui togliamo la responsabilità di salvare 
def try_ocr(file: Path) -> ConversionResult:
    if file.suffix.lower() != ".pdf":
        return ConversionResult(False, error="Non è un pdf")
    return convert_scanned_pdf(file)


def convert(source_path: str, extensions: Optional[list] = None,
            skip_selector=None,
            post_handlers: Optional[List[PostConversionHandler]] = None) -> None:
    extensions = extensions or config.extension
    
    file_explorer = FileExplorer(source_path)
    file_explorer.traverse()    

    converters = [Convert("tika"), Convert("markitdown")]

    for group in extensions:
        for file in file_explorer.files.get(group, []):
            # MMMMMMMMHHHHHHH
            # Esempio di filtro che agisce sul nome del file.
            # Non so ancora se questo è il suo posto.
            if skip_selector and skip_selector(file):
                log.info(f"File {file} ignorato da skip_selector")
                continue
            
            target_file = file.with_suffix(".md")

            if target_file.exists():
                log.debug(f"{target_file} presente, skip")
                continue

            result = convert_file(file, converters)

            if not result.success:
                result = try_ocr(file)
            
            if result.success:
                # Qui ho il file convertito. 
                # prima di chiamare il salvataggio posso attivare delle
                # sentinelle (Es, Rassegna Stampa evita AI)
                # Ricordati che in realtà gli handlers ritornano True/False
                # se mai dovesse servire
                for handler in post_handlers or []:
                    handler.handle(file, result)

                save_converted_file(file, result)
            else:
                log.error(f"Conversione fallita per {file}: {result.error}")