from abc import ABC, abstractmethod
from dataclasses import dataclass
from pathlib import Path
from typing import List, Optional, Tuple, Any
import json
import time
import logging

from gtagger.config import get_config
from gtagger.model import Document
config = get_config()
log = logging.getLogger(__name__)

PROMPT_DOCUMENT = """
Analizza il testo fornito e genera un output in formato JSON.

Assicurati che l'output sia preciso e ben strutturato. Restituisci solo l'oggetto JSON, senza ulteriori testi o spiegazioni.
Note:
- Un "comunicato stampa" è un testo redatto dall'ufficio stampa di VeronaFiere per informare su un fatto oppure su un avvenimento in maniera ufficiale.
- Un "comunicato stampa di terzi" è un testo redatto dall'ufficio stampa di un'azienda o di un ente pubblico per informare su un fatto oppure su un avvenimento in maniera ufficiale di Verona Fiere.
- Un "discorso" è un intervento specifico, tenuto da una singola persona che argomenta a favore di una tesi. E' organizzato per temi, sviluppa argomentazioni e può presentare dati a supporto.
- Una "intervista" è una forma di dialogo strutturato in cui una persona (l'intervistatore) pone delle domande a un'altra persona o a un gruppo di persone (l'intervistato/gli intervistati) con lo scopo di ottenere informazioni, opinioni, testimonianze o approfondimenti su un argomento specifico.
- Un "redazionale"  è un articolo o un testo di carattere giornalistico, che ha lo scopo di presentare un'azienda, un prodotto, un servizio o un evento in modo informativo e promozionale, ma con un tono e uno stile simili a quelli di un contenuto editoriale "neutro". Può essere anche un articolo di giornale inviato a riviste.
- Un "social" è un documento dove ogni paragrafo è un messaggio scritto per essere pubblicato su un social, per esempio Facebook o FB, Instagram o IG, Linkedin.
- Un "invito" è l'offerta fatta ad una persona, ente, azienda di partecipare ad una manifestazione o evento.
- Gli eventi sono classificati come "ARTVERONA, CORPORATE, EOS, FIERACAVALLI, FIERAGRICOLA, JOB&ORIENTA, LETEXPO, MARMOMAC, MCTER EXPO, MINERAL MAGGIO, MINERAL NOVEMBRE, MODEL EXPO, MOTORBIKE, PAVING, PROGETTO FUOCO, SAMOTER, SOL2EXPO, SPORT EXPO, VINITALY, AND THE CITY, VITA IN CAMPAGNA, WINE2WINE"
- Una "scaletta" è un elenco strutturato di attività che si svolgono durante un evento, hanno la struttura di programma o calendario che include orari, partecipanti, e altri dettagli necessari per la sua esecuzione.
- Un "programma" è l'elenco di tutte le attività di un evento, che si svolgono in più giornate o date.
"""


PROMPT_DATE = """Esamina la stringa allegata: 
 
genera un JSON ben fomattato con le seguenti informazioni disposte per colonna:

    campo "row" con la stringa esaminata

    campo "date"  con la data che  identifichi in formato ISO, usando XX per le parti non rilevate

    campo "persona": ["cognomi che identifichi nella stringa"]

    campo "info" è una stringa ottenuta così: elimina dalla stringa esaminata le informazioni che hai usato per la  data, i cognomi trovati e le seguenti parti:
    ["VINITALY", "FIERACAVALLI","FIERAGRICOLA","cs", "gb" , "gbcs", "riferimenti a lingua usata"] 
    
""" 




@dataclass
class ProcessingContext:
    """Contesto condiviso durante l'elaborazione di un documento"""
    source_file: Path
    target_file: Path
    tag_file: Path
    blacklist_file: Path
    content: str
    token_count: int
    is_blacklisted: bool

@dataclass 
class ProcessingResult:
    """Risultato dell'elaborazione di un documento"""
    document: Document  # Document object
    usage_data: Optional[dict] = None
    usage_filename: Optional[dict] = None
    filename_info: Optional[dict] = None

class DocumentProcessor(ABC):
    """Interfaccia base per processori di documenti"""
    
    @abstractmethod
    def can_process(self, context: ProcessingContext) -> bool:
        """Verifica se questo processor può gestire il documento"""
        pass
    
    @abstractmethod
    def process(self, context: ProcessingContext) -> ProcessingResult:
        """Elabora il documento e restituisce il risultato"""
        pass

class BlacklistedDocumentProcessor(DocumentProcessor):
    """Processore per documenti blacklistati (file .bl presente)"""
    
    def can_process(self, context: ProcessingContext) -> bool:
        return context.is_blacklisted
    
    def process(self, context: ProcessingContext) -> ProcessingResult:
        log.debug(f"Processing blacklisted document: {context.source_file}")
        
        from gtagger.model import Document  # Sostituisci con il tuo import
        
        # Se il file è blacklistato, nella black list c'e' anche   il tipo di documento
        # Metto a generale come default
        tipo_documento = "gen"
        with open(context.blacklist_file, "r") as f:
                for line in f:
                    stripped_line = line.strip()
                    if stripped_line.lower() == "rassegna stampa":
                        tipo_documento="rs"
                        break

        # Crea documento con valori di default per file blacklistati
        tagged_result = Document(
            persone_citate=[],
            relatori=[],
            data_evento="1970",
            tipo_documento=tipo_documento,  # Tipo specifico per blacklisted
            macroargomenti_trattati=[],
            lingua_utilizzata=""
        )
        
        return ProcessingResult(document=tagged_result)
class SmallDocumentProcessor(DocumentProcessor):
    """Processore per documenti piccoli (sotto soglia token)"""
    
    def __init__(self, min_tokens: int = 100):
        self.min_tokens = min_tokens
    
    def can_process(self, context: ProcessingContext) -> bool:
        return not context.is_blacklisted and context.token_count < self.min_tokens
    
    def process(self, context: ProcessingContext) -> ProcessingResult:
        log.debug(f"Processing small document: {context.source_file}")
        
        # Crea documento con valori di default
        from gtagger.model import Document  # Sostituisci con il tuo import
        
        tagged_result = Document(
            persone_citate=[],
            relatori=[],
            data_evento="1970",
            tipo_documento="img",
            macroargomenti_trattati=[],
            lingua_utilizzata=""
        )
        
        return ProcessingResult(document=tagged_result)

class LLMDocumentProcessor(DocumentProcessor):
    """Processore per documenti che richiedono elaborazione LLM"""
    
    def __init__(self, llm_invoker, prompts):
        self.llm = llm_invoker
        self.prompt_document = prompts.get('document')
        self.prompt_date = prompts.get('date')
    
    def can_process(self, context: ProcessingContext) -> bool:
        # Solo se non è blacklistato e ha abbastanza token
        return not context.is_blacklisted and context.token_count >= 100
    
    def process(self, context: ProcessingContext) -> ProcessingResult:
        log.debug(f"Processing with LLM: {context.source_file}")
        
        from gtagger.model import Document  # Sostituisci con il tuo import
        
        # Analisi principale del documento
        tagged_result, usage_data = self.llm.analyze(
            self.prompt_document, 
            context.content, 
            template=Document
        )
        
        self._validate_llm_response(tagged_result, usage_data)
        
        # Analisi della data dal nome file
        response, usage_filename = self.llm.analyze(
            self.prompt_date, 
            context.source_file.stem
        )
        
        self._validate_llm_response(response, usage_filename)
        
        # Normalizza risposta se è una lista
        if isinstance(response, list):
            response = response[0]
        
        log.debug(f"Date extraction result: {response}")
        
        return ProcessingResult(
            document=tagged_result,
            usage_data=usage_data,
            usage_filename=usage_filename,
            filename_info=response
        )
    
    def _validate_llm_response(self, result, usage):
        """Valida la risposta del LLM"""
        assert result is not None, "LLM returned None result"
        assert usage is not None, "LLM returned None usage data"

class DocumentPostProcessorChain:
    """Chain of responsibility per il post-processing"""
    
    def __init__(self, document_post_processor):
        self.post_processor = document_post_processor
        # Lista di metodi di post-processing da applicare in ordine
        self.processing_rules = [
            'apply_filename_type_rules',
            'apply_language_rules', 
            'apply_date_rules'
        ]
    
    def apply_all_rules(self, result: ProcessingResult, context: ProcessingContext):
        """Applica tutte le regole di post-processing in sequenza"""
        document = result.document
        
        # Applica regole dinamicamente dalla lista
        for rule_name in self.processing_rules:
            if hasattr(self.post_processor, rule_name):
                rule_method = getattr(self.post_processor, rule_name)
                
                if rule_name == 'apply_filename_type_rules':
                    rule_method(document, context.source_file.stem)
                elif rule_name == 'apply_language_rules':
                    rule_method(document, context.target_file)
                elif rule_name == 'apply_date_rules':
                    rule_method(document, context.source_file, result.filename_info)
                # Aggiungi altri pattern per nuove regole
            else:
                log.warning(f"Regola {rule_name} non trovata nel post-processor")
        
        # Imposta tipo documento di default se mancante
        if not document.tipo_documento:
            document.tipo_documento = "def"
        
        # Aggiungi metadati di utilizzo
        self._attach_usage_metadata(document, result)
    
    def _attach_usage_metadata(self, document, result: ProcessingResult):
        """Attacca i metadati di utilizzo al documento"""
        if hasattr(document, '_usage_data') and result.usage_data:
            document._usage_data = result.usage_data
        if hasattr(document, '_usage_filename') and result.usage_filename:
            document._usage_filename = result.usage_filename
        if hasattr(document, '_filename_info') and result.filename_info:
            document._filename_info = result.filename_info
    
    def add_processing_rule(self, rule_name: str, position: Optional[int] = None):
        """Aggiunge una nuova regola di processing"""
        if position is None:
            self.processing_rules.append(rule_name)
        else:
            self.processing_rules.insert(position, rule_name)

class FileValidator:
    """Valida i file prima dell'elaborazione"""
    # Gli stati che contano
    # |  BL  |  MD  |  JSON  | Process | is_blacklisted |
    # |      |   F  |        |    F    |                | 
    # |      |      |   T    |    F    |                |
    # |   T  |   T  |   F    |    T    |       T        |
    # |   F  |   T  |   F    |    T    |       F        |
    @staticmethod
    def validate_files(source_file: Path) -> Tuple[Path, Path, Path, bool, bool]:
        """Valida i file e restituisce i path necessari
        - file da inviare ad AI
        - file cintenitore dei risultati
        - file di black list
        - flag: True il file va passato ai processori
        - flag: True il file è "blacklisted"

        Vero che potrei usare solo il primo flag per elaborare o meno, ma
        devo tenere conto anche dell'esistenza del file json per mantenere coenza.
        Ho mantenuto il nome target_file per coerenza con "process_document"      
        """

        target_file = source_file.with_suffix(".md")
        tag_file = source_file.with_suffix(".json")
        blacklist_file = source_file.with_suffix(".bl")
        
        # Vincolo: deve esistere un file convertito
        if not target_file.exists():
            log.warning(f"File non convertito trovato: {source_file}")
            return target_file, tag_file, blacklist_file, False, False
        
        # Vincolo: il file non è già stato processato
        if tag_file.exists():
            log.debug(f"JSON {tag_file} già presente: skip")
            return target_file, tag_file, blacklist_file, False, False
        
        # Vincolo: il file 
        # TODO: In futuro sostituire con query SQLite3
        is_blacklisted = blacklist_file.exists()
        if is_blacklisted:
            log.debug(f"File blacklistato trovato: {blacklist_file}")
        
        return target_file, tag_file, blacklist_file, True, is_blacklisted

##### Orchestratore: decide il destino di un documento  
#  - lo sottopone al validatore, che ne estrae le caratteristiche necessarie alla decisione
#  - prepara il contesto
#  - lo sottopone ai processori, che in base alle informazioni di validazione operano
#    su di esso nel modo opportuno o non fanno nulla (agisce un processore per documento)
class DocumentProcessingOrchestrator:
    """Orchestratore principale per l'elaborazione dei documenti"""
    
    def __init__(self, llm_invoker, prompts, document_post_processor, min_tokens: int = 100):
        # Ordine importante: processori più specifici prima
        self.processors = [
            BlacklistedDocumentProcessor(),
            SmallDocumentProcessor(min_tokens),
            LLMDocumentProcessor(llm_invoker, prompts)
        ]
        self.post_processor_chain = DocumentPostProcessorChain(document_post_processor)
        self.file_validator = FileValidator()
    
    def process_document(self, source_file: Path) -> bool:
        """Elabora un singolo documento"""
        log.debug(f"Controllo file: {source_file}")
        
        # Validazione file, attraverso cui stabilisco anche se saltare lo step AI
        target_file, tag_file, blacklist_file, should_process, is_blacklisted = self.file_validator.validate_files(source_file)
        if not should_process:
            return False
        
        # Lettura contenuto
        try:
            with open(target_file, "r") as f:
                content = f.read()
        except Exception as e:
            log.error(f"Errore lettura file {target_file}: {e}")
            return False
        
        # Creazione contesto
        context = ProcessingContext(
            source_file=source_file,
            target_file=target_file,
            tag_file=tag_file,
            blacklist_file=blacklist_file,
            content=content,
            token_count=len(content.split()),
            is_blacklisted=is_blacklisted
        )
        
        # Selezione e esecuzione processor
        for processor in self.processors:
            if processor.can_process(context):
                try:
                    result = processor.process(context)
                    self.post_processor_chain.apply_all_rules(result, context)
                    self._save_result(result.document, tag_file)
                    
                    # Log specifico per tipo di processing
                    processor_type = type(processor).__name__
                    log.info(f"Documento elaborato con {processor_type}: {source_file}")
                    return True
                except Exception as e:
                    log.error(f"Errore elaborazione {source_file} con {type(processor).__name__}: {e}")
                    return False
        
        log.error(f"Nessun processor disponibile per {source_file}")
        return False
    
    def _save_result(self, document, tag_file: Path):
        """Salva il risultato su file"""
        with open(tag_file, "w") as f:
            json.dump(document.full_serialize(), f)
        log.info(f"File creato: {tag_file}")
        
        # TODO: Rimuovere questo sleep quando non più necessario
        time.sleep(3)

##### AI 
#  - crea il contenitore dei percorsi file (FileExplorer)
#  - Istanzia il postprocessore (prima del ciclo, quindi una volta)
#  - Istanzia l'orchestratore (una istanza prima del ciclo)
#  - Per ogni documento (tutti i documenti)
#  - invia il documento all'orchestratore
#  - raccogli statistiche (un di più)
def ai(path):
    """Funzione principale rifattorizzata"""
    from gtagger.llm_invoker import LLMInvoker
    from gtagger.document_processing import DocumentPostProcessor
    from gtagger.file_explorer import FileExplorer
    
    # Setup
    file_explorer = FileExplorer(path)
    file_explorer.traverse()
    
    llm = LLMInvoker()
    prompts = {
        'document': PROMPT_DOCUMENT,
        'date': PROMPT_DATE
    }
    document_post_processor = DocumentPostProcessor()
    
    # Creazione orchestratore
    orchestrator = DocumentProcessingOrchestrator(
        llm_invoker=llm,
        prompts=prompts,
        document_post_processor=document_post_processor,
        min_tokens=100
    )
    
    # Elaborazione file
    processed_count = 0
    total_count = 0
    blacklisted_count = 0
    small_doc_count = 0
    llm_processed_count = 0
    
    # Inizialmente filtravo tutto per "*.md"
    # è meglio delegare la responsabilità per le decisioni
    for group in set(config.extension):
        for source_file in file_explorer.files.get(group, []):
            total_count += 1
            
            # Per statistiche
            blacklist_file = source_file.with_suffix(".bl")
            target_file = source_file.with_suffix(".md")
            
            if blacklist_file.exists():
                blacklisted_count += 1
            elif target_file.exists():
                with open(target_file, "r") as f:
                    content = f.read()
                if len(content.split()) < 100:
                    small_doc_count += 1
                else:
                    llm_processed_count += 1
            # fine raccolta statistiche

            if orchestrator.process_document(source_file):
                processed_count += 1
    
    log.info(f"Elaborazione completata: {processed_count}/{total_count} file processati")
    log.info(f"Statistiche: {blacklisted_count} blacklisted, {small_doc_count} piccoli, {llm_processed_count} elaborati con LLM")